@@ -56,10 +56,10 @@
 
 #define GETDNS_PORT_ZERO 0
 #define GETDNS_PORT_DNS 53
-#define GETDNS_PORT_DNS_OVER_TLS 1021
+#define GETDNS_PORT_DNS_OVER_TLS 853
 #define GETDNS_STR_PORT_ZERO "0"
 #define GETDNS_STR_PORT_DNS "53"
-#define GETDNS_STR_PORT_DNS_OVER_TLS "1021"
+#define GETDNS_STR_PORT_DNS_OVER_TLS "853"
 
 void *plain_mem_funcs_user_arg = MF_PLAIN;
 
@@ -567,8 +567,10 @@ _getdns_upstream_shutdown(getdns_upstream *upstream)
 		upstream->tcp.write_error = 0;
 	upstream->writes_done = 0;
 	upstream->responses_received = 0;
-	if (upstream->tls_hs_state != GETDNS_HS_FAILED)
+	if (upstream->tls_hs_state != GETDNS_HS_FAILED) {
 		upstream->tls_hs_state = GETDNS_HS_NONE;
+		upstream->tls_auth_failed = 0;
+	}
 	/* Now TLS stuff*/
 	if (upstream->tls_obj != NULL) {
 		SSL_shutdown(upstream->tls_obj);
@@ -579,6 +581,27 @@ _getdns_upstream_shutdown(getdns_upstream *upstream)
 		close(fd);
 }
 
+static int
+tls_is_in_transports_list(getdns_context *context) {
+	for (int i=0; i< context->dns_transport_count;i++) {
+		if (context->dns_transports[i] == GETDNS_TRANSPORT_TLS ||
+		    context->dns_transports[i] == GETDNS_TRANSPORT_STARTTLS)
+			return 1;
+	}
+	return 0;
+}
+
+static int
+tls_only_is_in_transports_list(getdns_context *context) {
+	if (context->dns_transport_count != 1)
+		return 0;
+	if (context->dns_transports[0] == GETDNS_TRANSPORT_TLS ||
+		context->dns_transports[0] == GETDNS_TRANSPORT_STARTTLS)
+			return 1;
+	return 0;
+}
+
+
 static int
 net_req_query_id_cmp(const void *id1, const void *id2)
 {
@@ -606,6 +629,8 @@ upstream_init(getdns_upstream *upstream,
 	upstream->starttls_req = NULL;
 	upstream->transport = GETDNS_TRANSPORT_TCP;
 	upstream->tls_hs_state = GETDNS_HS_NONE;
+	upstream->tls_auth_failed = 0;
+	upstream->tls_auth_name[0] = '\0';
 	upstream->tcp.write_error = 0;
 	upstream->loop = NULL;
 	(void) getdns_eventloop_event_init(
@@ -677,7 +702,7 @@ set_os_defaults(struct getdns_context *context)
 			token = parse + strcspn(parse, " \t\r\n");
 			*token = 0;
 
-			(void) strcpy(domain, parse);
+			(void) strlcpy(domain, parse, sizeof(domain));
 
 		} else if (strncmp(parse, "search", 6) == 0) {
 			parse += 6;
@@ -875,6 +900,8 @@ getdns_context_create_with_extended_memory_functions(
 	result->edns_maximum_udp_payload_size = -1;
 	if ((r = create_default_dns_transports(result)))
 		goto error;
+	result->tls_auth = GETDNS_AUTHENTICATION_NONE; 
+	result->tls_auth_min = GETDNS_AUTHENTICATION_NONE;
 	result->limit_outstanding_queries = 0;
 	result->return_dnssec_status = GETDNS_EXTENSION_FALSE;
 
@@ -1158,6 +1185,10 @@ getdns_context_set_resolution_type(struct getdns_context *context,
     if (value != GETDNS_RESOLUTION_STUB && value != GETDNS_RESOLUTION_RECURSING) {
         return GETDNS_RETURN_CONTEXT_UPDATE_FAIL;
     }
+#ifndef HAVE_LIBUNBOUND
+    if (value == GETDNS_RESOLUTION_RECURSING)
+	    return GETDNS_RETURN_NOT_IMPLEMENTED;
+#endif
 #ifndef STUB_NATIVE_DNSSEC
     if (context->resolution_type_set != 0) {
         /* already setup */
@@ -1219,18 +1250,23 @@ getdns_set_base_dns_transports(
 	if (!context || transport_count == 0 || transports == NULL)
 		return GETDNS_RETURN_INVALID_PARAMETER;
 
+	/* Check for valid transports and that they are used only once*/
+	int u=0,t=0,l=0,s=0;
 	for(i=0; i<transport_count; i++)
 	{
-		if( transports[i] != GETDNS_TRANSPORT_UDP
-		 && transports[i] != GETDNS_TRANSPORT_TCP
-		 && transports[i] != GETDNS_TRANSPORT_TLS
-		 && transports[i] != GETDNS_TRANSPORT_STARTTLS)
-			return GETDNS_RETURN_INVALID_PARAMETER;
+		switch (transports[i]) {
+			case GETDNS_TRANSPORT_UDP:       u++; break;
+			case GETDNS_TRANSPORT_TCP:       t++; break;
+			case GETDNS_TRANSPORT_TLS:       l++; break;
+			case GETDNS_TRANSPORT_STARTTLS:  s++; break;
+			default: return GETDNS_RETURN_INVALID_PARAMETER;
+		}
 	}
+	if ( u>1 || t>1 || l>1 || s>1)
+		return GETDNS_RETURN_INVALID_PARAMETER;
 	
 	if (!(new_transports = GETDNS_XMALLOC(context->my_mf,
-				getdns_transport_list_t, transport_count)))
-
+		getdns_transport_list_t, transport_count)))
 		return GETDNS_RETURN_CONTEXT_UPDATE_FAIL;
 
 	if (context->dns_transports)
@@ -1241,7 +1277,6 @@ getdns_set_base_dns_transports(
 	memcpy(context->dns_transports, transports,
 	    transport_count * sizeof(getdns_transport_list_t));
 	context->dns_transport_count = transport_count;
-	dispatch_updated(context, GETDNS_CONTEXT_CODE_NAMESPACES);
 
 	return GETDNS_RETURN_GOOD;
 }
@@ -1327,7 +1362,7 @@ getdns_context_set_dns_transport(
 	context->dns_transport_count = count;
 	context->dns_transports = new_transports;
 
-	switch (value) {
+	switch ((int)value) {
 	    case GETDNS_TRANSPORT_UDP_FIRST_AND_FALL_BACK_TO_TCP:
 	        context->dns_transports[0] = GETDNS_TRANSPORT_UDP;
 	        context->dns_transports[1] = GETDNS_TRANSPORT_TCP;
@@ -1390,6 +1425,26 @@ getdns_context_set_dns_transport_list(getdns_context *context,
     return GETDNS_RETURN_GOOD;
 }               /* getdns_context_set_dns_transport_list */
 
+/*
+ * getdns_context_set_tls_authentication
+ *
+ */
+getdns_return_t
+getdns_context_set_tls_authentication(getdns_context *context,
+                                      getdns_tls_authentication_t value)
+{
+    RETURN_IF_NULL(context, GETDNS_RETURN_INVALID_PARAMETER);
+    if (value != GETDNS_AUTHENTICATION_NONE && 
+        value != GETDNS_AUTHENTICATION_HOSTNAME) {
+        return GETDNS_RETURN_CONTEXT_UPDATE_FAIL;
+    }
+    context->tls_auth = value;
+
+    dispatch_updated(context, GETDNS_CONTEXT_CODE_TLS_AUTHENTICATION);
+
+    return GETDNS_RETURN_GOOD;
+}               /* getdns_context_set_tls_authentication_list */
+
 static void
 set_ub_limit_outstanding_queries(struct getdns_context* context, uint16_t value) {
     /* num-queries-per-thread */
@@ -1665,6 +1720,7 @@ getdns_context_set_upstream_recursive_servers(struct getdns_context *context,
 		getdns_dict *dict;
 		getdns_bindata *address_type;
 		getdns_bindata *address_data;
+		getdns_bindata *tls_auth_name;
 		struct sockaddr_storage  addr;
 
 		getdns_bindata *scope_id;
@@ -1738,6 +1794,17 @@ getdns_context_set_upstream_recursive_servers(struct getdns_context *context,
 			upstream->addr.ss_family = addr.ss_family;
 			upstream_init(upstream, upstreams, ai);
 			upstream->transport = getdns_upstream_transports[j];
+			if (getdns_upstream_transports[j] == GETDNS_TRANSPORT_TLS ||
+			    getdns_upstream_transports[j] == GETDNS_TRANSPORT_STARTTLS) {
+				if ((r = getdns_dict_get_bindata(
+					dict, "tls_auth_name", &tls_auth_name)) == GETDNS_RETURN_GOOD) {
+					/*TODO: VALIDATE THIS STRING!*/
+					memcpy(upstream->tls_auth_name,
+					       (char *)tls_auth_name->data,
+						tls_auth_name->size);
+					upstream->tls_auth_name[tls_auth_name->size] = '\0';
+				}
+			}
 			upstreams->count++;
 			freeaddrinfo(ai);
 		}
@@ -2133,24 +2200,43 @@ _getdns_context_prepare_for_resolution(struct getdns_context *context,
     }
 
 	/* Transport can in theory be set per query in stub mode */
-	if (context->resolution_type == GETDNS_RESOLUTION_STUB) {
-		/*TODO[TLS]: Check if TLS is in the list of transports.*/
+	if (context->resolution_type == GETDNS_RESOLUTION_STUB && 
+		tls_is_in_transports_list(context) == 1) {
 		if (context->tls_ctx == NULL) {
-#ifdef HAVE_LIBTLS1_2
+#ifdef HAVE_TLS_v1_2
 			/* Create client context, use TLS v1.2 only for now */
 			context->tls_ctx = SSL_CTX_new(TLSv1_2_client_method());
-#endif
 			if(context->tls_ctx == NULL)
 				return GETDNS_RETURN_BAD_CONTEXT;
+			/* Be strict and only use the cipher suites recommended in RFC7525
+			   Unless we later fallback to oppotunistic. */
+			const char* const PREFERRED_CIPHERS = "EECDH+aRSA+AESGCM:EDH+aRSA+AESGCM";
+			if (!SSL_CTX_set_cipher_list(context->tls_ctx, PREFERRED_CIPHERS))
+				return GETDNS_RETURN_BAD_CONTEXT;
+			if (!SSL_CTX_set_default_verify_paths(context->tls_ctx))
+				return GETDNS_RETURN_BAD_CONTEXT;
+#else
+			if (tls_only_is_in_transports_list(context) == 1)
+				return GETDNS_RETURN_BAD_CONTEXT;
+			/* A null tls_ctx will make TLS fail and fallback to the other
+			   transports will kick-in.*/
+#endif
+		}
+		if (tls_only_is_in_transports_list(context) == 1 && 
+		    context->tls_auth == GETDNS_AUTHENTICATION_HOSTNAME) {
+			context->tls_auth_min = GETDNS_AUTHENTICATION_HOSTNAME;
+			/* TODO: If no auth data provided for any upstream, fail here */
+		}
+		else {
+			context->tls_auth_min = GETDNS_AUTHENTICATION_NONE;
 		}
 	}
+
 	/* Block use of STARTTLS/TLS ONLY in recursive mode as it won't work */
     /* Note: If TLS is used in recursive mode this will try TLS on port
-     * 53 so it is blocked here.  So is STARTTLS only at the moment. */
+     * 53 so it is blocked here.  So is 'STARTTLS only' at the moment. */
 	if (context->resolution_type == GETDNS_RESOLUTION_RECURSING &&
-		context->dns_transport_count == 1 && 
-	    (context->dns_transports[0] == GETDNS_TRANSPORT_TLS ||
-	     context->dns_transports[0] == GETDNS_TRANSPORT_STARTTLS))
+		tls_only_is_in_transports_list(context) == 1)
 		return GETDNS_RETURN_BAD_CONTEXT;
 
 	if (context->resolution_type_set == context->resolution_type)
@@ -2306,8 +2392,10 @@ getdns_context_get_num_pending_requests(struct getdns_context* context,
 		context->extension->vmt->run_once(context->extension, 0);
 
 	/* TODO: Remove this when next_timeout is gone */
-	_getdns_handle_timeouts(context->mini_event.base,
-	    &context->mini_event.time_tv, next_timeout ? next_timeout : &dispose);
+	if (context->extension == &context->mini_event.loop)
+		_getdns_handle_timeouts(context->mini_event.base,
+		    &context->mini_event.time_tv,
+		    next_timeout ? next_timeout : &dispose);
 
 	return context->outbound_requests.count;
 }
@@ -2332,8 +2420,12 @@ getdns_context_process_async(struct getdns_context* context)
 void
 getdns_context_run(getdns_context *context)
 {
-	if (getdns_context_get_num_pending_requests(context, NULL) > 0 &&
-	    !getdns_context_process_async(context))
+	if (context->extension == &context->mini_event.loop) {
+		if (getdns_context_get_num_pending_requests(context, NULL) > 0 &&
+		    !getdns_context_process_async(context))
+			context->extension->vmt->run(context->extension);
+	}
+	else
 		context->extension->vmt->run(context->extension);
 }
 
@@ -2384,15 +2476,15 @@ getdns_context_detach_eventloop(struct getdns_context* context)
 }
 
 getdns_return_t
-getdns_context_set_eventloop(struct getdns_context* context, getdns_eventloop* loop)
+getdns_context_set_eventloop(getdns_context* context, getdns_eventloop* loop)
 {
-	RETURN_IF_NULL(context, GETDNS_RETURN_INVALID_PARAMETER);
-	RETURN_IF_NULL(loop   , GETDNS_RETURN_INVALID_PARAMETER);
-
-	getdns_return_t r = getdns_context_detach_eventloop(context);
-	if (r != GETDNS_RETURN_GOOD)
-		return r;
+	if (!context || !loop)
+		return GETDNS_RETURN_INVALID_PARAMETER;
 
+	if (context->extension) {
+		cancel_outstanding_requests(context, 1);
+		context->extension->vmt->cleanup(context->extension);
+	}
 	context->extension = loop;
 	return GETDNS_RETURN_GOOD;
 }
