@@ -32,6 +32,8 @@
  */
 
 #include <openssl/err.h>
+#include <openssl/conf.h>
+#include <openssl/x509v3.h>
 #include "config.h"
 #include <fcntl.h>
 #include "stub.h"
@@ -181,7 +183,7 @@ match_and_process_server_cookie(
 {
 	_getdns_rr_iter rr_iter_storage, *rr_iter;
 	uint8_t *pos;
-	uint16_t rdata_len, opt_code, opt_len;
+	uint16_t rdata_len, opt_code = 0, opt_len = 0;
 
 	/* Search for the OPT RR (if any) */
 	for ( rr_iter = _getdns_rr_iter_init(&rr_iter_storage
@@ -562,7 +564,7 @@ upstream_tls_timeout_cb(void *userarg)
 	tls_cleanup(upstream);
 
 	/* Need to handle the case where the far end doesn't respond to a
-	 * TCP SYN and doesn't do a reset (as is the case with e.g. 8.8.8.8@1021).
+	 * TCP SYN and doesn't do a reset (as is the case with e.g. 8.8.8.8@853).
 	 * For that case the socket never becomes writable so doesn't trigger any
 	 * callbacks. If so then clear out the queue in one go.*/
 	int ret;
@@ -589,7 +591,7 @@ stub_tls_timeout_cb(void *userarg)
 	tls_cleanup(upstream);
 
 	/* Need to handle the case where the far end doesn't respond to a
-	 * TCP SYN and doesn't do a reset (as is the case with e.g. 8.8.8.8@1021).
+	 * TCP SYN and doesn't do a reset (as is the case with e.g. 8.8.8.8@853).
 	 * For that case the socket never becomes writable so doesn't trigger any
 	 * callbacks. If so then clear out the queue in one go.*/
 	int ret;
@@ -818,13 +820,59 @@ tls_failed(getdns_upstream *upstream)
 	/* No messages should be scheduled onto an upstream in this state */
 	return ((upstream->transport == GETDNS_TRANSPORT_TLS ||
 	         upstream->transport == GETDNS_TRANSPORT_STARTTLS) &&
-	         upstream->tls_hs_state == GETDNS_HS_FAILED) ? 1: 0;
+	         upstream->tls_hs_state == GETDNS_HS_FAILED) ? 1 : 0;
+}
+
+static int
+tls_auth_status_ok(getdns_upstream *upstream, getdns_network_req *netreq) {
+	return (netreq->tls_auth_min == GETDNS_AUTHENTICATION_HOSTNAME &&
+		    upstream->tls_auth_failed) ? 0 : 1;
+}
+
+int
+tls_verify_callback(int preverify_ok, X509_STORE_CTX *ctx)
+{
+#if defined(STUB_DEBUG) && STUB_DEBUG
+	int     err;
+	const char * err_str;
+
+	err = X509_STORE_CTX_get_error(ctx);
+	err_str = X509_verify_cert_error_string(err);
+	DEBUG_STUB("--- %s, VERIFY RESULT: %s\n", __FUNCTION__, err_str);
+#endif
+	/*Always proceed without changing result*/
+	return preverify_ok;
+}
+
+int
+tls_verify_callback_with_fallback(int preverify_ok, X509_STORE_CTX *ctx)
+{
+#ifdef X509_V_ERR_HOSTNAME_MISMATCH
+	int     err;
+# if defined(STUB_DEBUG) && STUB_DEBUG
+	const char * err_str;
+# endif
+	
+	err = X509_STORE_CTX_get_error(ctx);
+# if defined(STUB_DEBUG) && STUB_DEBUG
+	err_str = X509_verify_cert_error_string(err);
+	DEBUG_STUB("--- %s, VERIFY RESULT: (%d) \"%s\"\n", __FUNCTION__, err, err_str);
+# endif
+	/*Proceed if error is hostname mismatch*/
+	if (err == X509_V_ERR_HOSTNAME_MISMATCH) {
+		DEBUG_STUB("--- %s, PROCEEDING WITHOUT HOSTNAME VALIDATION!!\n", __FUNCTION__);
+		return 1;
+	}
+	else
+#endif
+		return preverify_ok;
 }
 
 static SSL*
-tls_create_object(getdns_context *context, int fd)
+tls_create_object(getdns_dns_req *dnsreq, int fd, getdns_upstream *upstream)
 {
 	/* Create SSL instance */
+	getdns_context *context = dnsreq->context;
 	if (context->tls_ctx == NULL)
 		return NULL;
 	SSL* ssl = SSL_new(context->tls_ctx);
@@ -835,6 +883,53 @@ tls_create_object(getdns_context *context, int fd)
 		SSL_free(ssl);
 		return NULL;
 	}
+
+	/* NOTE: this code will fallback on a given upstream, without trying
+	   authentication on other upstreams first. This is non-optimal and but avoids
+	   multiple TLS handshakes before getting a usable connection. */
+
+	/* If we have a hostname, always use it */
+	if (upstream->tls_auth_name[0] != '\0') {
+		/*Request certificate for the auth_name*/
+		SSL_set_tlsext_host_name(ssl, upstream->tls_auth_name);
+#ifdef HAVE_SSL_HN_AUTH
+		/* Set up native OpenSSL hostname verification*/
+		X509_VERIFY_PARAM *param;
+		param = SSL_get0_param(ssl);
+		X509_VERIFY_PARAM_set_hostflags(param, X509_CHECK_FLAG_NO_PARTIAL_WILDCARDS);
+		X509_VERIFY_PARAM_set1_host(param, upstream->tls_auth_name, 0);
+		DEBUG_STUB("--- %s, HOSTNAME VERIFICATION REQUESTED \n", __FUNCTION__);
+#else
+		if (dnsreq->netreqs[0]->tls_auth_min == GETDNS_AUTHENTICATION_HOSTNAME) {
+			/* TODO: Trigger post-handshake custom validation*/
+			DEBUG_STUB("--- %s, ERROR: Authentication functionality not available\n", __FUNCTION__);
+			upstream->tls_hs_state = GETDNS_HS_FAILED;
+			upstream->tls_auth_failed = 1;
+			return NULL;
+		}
+#endif
+		/* Allow fallback to oppotunisitc if settings permit it*/
+		if (dnsreq->netreqs[0]->tls_auth_min == GETDNS_AUTHENTICATION_HOSTNAME)
+			SSL_set_verify(ssl, SSL_VERIFY_PEER, tls_verify_callback);
+		else {
+			SSL_set_verify(ssl, SSL_VERIFY_NONE, tls_verify_callback_with_fallback);
+			SSL_set_cipher_list(ssl, "DEFAULT");
+		}
+	} else {
+		/* Lack of host name is OK unless only authenticated TLS is specified*/
+		if (dnsreq->netreqs[0]->tls_auth_min == GETDNS_AUTHENTICATION_HOSTNAME) {
+			DEBUG_STUB("--- %s, ERROR: No host name provided for authentication\n", __FUNCTION__);
+			upstream->tls_hs_state = GETDNS_HS_FAILED;
+			upstream->tls_auth_failed = 1;
+			return NULL;
+		} else {
+			DEBUG_STUB("--- %s, PROCEEDING WITHOUT HOSTNAME VALIDATION!!\n", __FUNCTION__);
+			upstream->tls_auth_failed = 1;
+			SSL_set_verify(ssl, SSL_VERIFY_NONE, tls_verify_callback_with_fallback);
+			SSL_set_cipher_list(ssl, "DEFAULT");
+		}
+	}
+	
 	SSL_set_connect_state(ssl);
 	(void) SSL_set_mode(ssl, SSL_MODE_AUTO_RETRY);
 	return ssl;
@@ -889,10 +984,16 @@ tls_do_handshake(getdns_upstream *upstream)
 				upstream->tls_hs_state = GETDNS_HS_WRITE;
 				return STUB_TCP_AGAIN;
 			default:
+				DEBUG_STUB("--- %s %s %d\n", __FUNCTION__, "Handshake failed: ", want);
 				return tls_cleanup(upstream);
 	   }
 	}
 	upstream->tls_hs_state = GETDNS_HS_DONE;
+	r = SSL_get_verify_result(upstream->tls_obj);
+#ifdef X509_V_ERR_HOSTNAME_MISMATCH
+	if (r == X509_V_ERR_HOSTNAME_MISMATCH)
+#endif
+		upstream->tls_auth_failed = 1;
 	/* Reset timeout on success*/
 	GETDNS_CLEAR_EVENT(upstream->loop, &upstream->event);
 	upstream->event.read_cb = NULL;
@@ -1033,6 +1134,8 @@ stub_tls_write(getdns_upstream *upstream, getdns_tcp_state *tcp,
 	int q = tls_connected(upstream);
 	if (q != 0)
 		return q;
+	if (!tls_auth_status_ok(upstream, netreq))
+		return STUB_TLS_SETUP_ERROR;
 
 	/* Do we have remaining data that we could not write before?  */
 	if (! tcp->write_buf) {
@@ -1301,8 +1404,9 @@ upstream_read_cb(void *userarg)
 		if (netreq->owner == upstream->starttls_req) {
 			dnsreq = netreq->owner;
 			if (is_starttls_response(netreq)) {
-				upstream->tls_obj = tls_create_object(dnsreq->context,
-				                                      upstream->fd);
+				upstream->tls_obj = tls_create_object(dnsreq,
+				                                      upstream->fd,
+				                                      upstream);
 				if (upstream->tls_obj == NULL) 
 					upstream->tls_hs_state = GETDNS_HS_FAILED;
 				upstream->tls_hs_state = GETDNS_HS_WRITE;
@@ -1438,7 +1542,8 @@ netreq_upstream_write_cb(void *userarg)
 
 static int
 upstream_transport_valid(getdns_upstream *upstream,
-                        getdns_transport_list_t transport)
+                        getdns_transport_list_t transport,
+                        getdns_network_req *netreq)
 {
 	/* Single shot UDP, uses same upstream as plain TCP. */
 	if (transport == GETDNS_TRANSPORT_UDP)
@@ -1459,7 +1564,7 @@ upstream_transport_valid(getdns_upstream *upstream,
 	/* Otherwise, transport must match, and not have failed */
 	if (upstream->transport != transport)
 		return 0;
-	if (tls_failed(upstream))
+	if (tls_failed(upstream) || !tls_auth_status_ok(upstream, netreq))
 		return 0;
 	return 1;
 }
@@ -1487,7 +1592,7 @@ upstream_select(getdns_network_req *netreq, getdns_transport_list_t transport)
 	DEBUG_STUB(" current upstream: %d of %d \n",(int)i, (int)upstreams->count);
 	do {
 		if (upstreams->upstreams[i].to_retry > 0 &&
-		    upstream_transport_valid(&upstreams->upstreams[i], transport)) {
+		    upstream_transport_valid(&upstreams->upstreams[i], transport, netreq)) {
 			upstreams->current = i;
 			DEBUG_STUB(" selected upstream: %d\n",(int)i);
 			return &upstreams->upstreams[i];
@@ -1499,11 +1604,11 @@ upstream_select(getdns_network_req *netreq, getdns_transport_list_t transport)
 	upstream = upstreams->upstreams;
 	for (i = 0; i < upstreams->count; i++)
 		if (upstreams->upstreams[i].back_off < upstream->back_off &&
-			upstream_transport_valid(&upstreams->upstreams[i], transport))
+			upstream_transport_valid(&upstreams->upstreams[i], transport, netreq))
 			upstream = &upstreams->upstreams[i];
 
 	/* Need to check again that the transport is valid */
-	if (!upstream_transport_valid(upstream, transport)) {
+	if (!upstream_transport_valid(upstream, transport, netreq)) {
 		DEBUG_STUB(" ! No valid upstream available\n");
 		return NULL;
 	}
@@ -1542,7 +1647,7 @@ upstream_connect(getdns_upstream *upstream, getdns_transport_list_t transport,
 			return upstream->fd;
 		fd = tcp_connect(upstream, transport);
 		if (fd == -1) return -1;
-		upstream->tls_obj = tls_create_object(dnsreq->context, fd);
+		upstream->tls_obj = tls_create_object(dnsreq, fd, upstream);
 		if (upstream->tls_obj == NULL) {
 			close(fd);
 			return -1;
@@ -1585,6 +1690,7 @@ find_upstream_for_specific_transport(getdns_network_req *netreq,
                              getdns_transport_list_t transport,
                              int *fd)
 {
+	// TODO[TLS]: Need to loop over upstreams here!! 
 	getdns_upstream *upstream = upstream_select(netreq, transport);
 	if (!upstream)
 		return NULL;
