@@ -35,22 +35,22 @@
 
 const char *getdns_get_version(void)
 {
-	return "0.3.3";
+	return "0.5.0";
 }
 
 uint32_t getdns_get_version_number(void)
 {
-	return 0x00030300;
+	return 0x00050000;
 }
 
 const char *getdns_get_api_version(void)
 {
-	return "July 2015";
+	return "October 2015";
 }
 
 uint32_t getdns_get_api_version_number(void)
 {
-	return 0x07df0700;
+	return 0x07df0a00;
 }
 
 /* version.c */
